package renegade.gfx.win32.ico;

import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

import renegade.gfx.io.*;

public class Win32Icon extends BufferedImage{
	private static int INFO_HEADER_SIZE = 40;
	// attributes
	private int width,height,colorCount,bitCount,planes,byteSize,offset;
	private int infoSize,infoWidth,infoHeight,infoPlanes,infoBitCount,infoCompression,infoImageSize;
	private int infoXPixelsPerM,infoYPixelsPerM,infoColorsUsed,infoColorsImportant;
	private File iconFile;
	private Color[] rgbPallete;
	int[] xorData;
	int[] andData;
	//
	private boolean rendered;

	/* Constructors */
	private Win32Icon(int width,int height){
		super(width,height,TYPE_INT_ARGB);
		rendered = false;
		rgbPallete = null;
		xorData = null;
	}
	
	/* overriden methods */
	public String toString(){
		return ("Icon (" + iconFile.toString() + "), " + width + "x" + height + "x" + colorCount + ",offset:" + offset + " ,size:" + byteSize);
	}
	
	/* accessors & mutators */
	//
	public int getWidth(){
		return width;
	}
	
	public int getHeight(){
		return height;
	}
	
	public int getColorCount(){
		return colorCount;
	}
	
	public int getPlanes(){
		return planes;
	}
	
	public int getSize(){
		return byteSize;
	}
	
	public int getImageSize(){
		return byteSize - INFO_HEADER_SIZE;
	}
	
	public int getOffset(){
		return offset;
	}
	
	public int getBitCount(){
		return bitCount;
	}
	
	public Color[] getPallete(){
		return rgbPallete;
	}
	
	public String getPalleteStr(){
		StringBuffer strbuf = new StringBuffer();
		strbuf.append("Found ").append(colorCount).append(" Colors\n");
		for(int counter = 0;counter < colorCount;counter++){
			strbuf.append("[").append(counter).append("] = ");
			strbuf.append("red: ").append(rgbPallete[counter].getRed()).append(", ");
			strbuf.append("green: ").append(rgbPallete[counter].getGreen()).append(", ");
			strbuf.append("blue: ").append(rgbPallete[counter].getBlue()).append("\n");
		}
		
		return strbuf.toString();
	}
	
	// get & set Info Header
	public int getInfoSize(){
		return infoSize;
	}
	
	public int getInfoWidth(){
		return infoWidth;
	}
	
	public int getInfoHeight(){
		return infoHeight;
	}
	
	public int getInfoBitCount(){
		return infoBitCount;
	}
	
	/* other functions */
	private int blendPixel(int x,int y,int pixel,Color c,float a){
		Color col = new Color(pixel);
		int new_red = (int)(a*col.getRed() + (1 - a)*c.getRed());
		int new_green = (int)(a*col.getGreen() + (1 - a)*c.getGreen());
		int new_blue = (int)(a*col.getBlue() + (1 - a)*c.getBlue());
		int alpha = 255;
		switch(andData[y*width + x]){
			case 0:
				alpha = 255;
				break;
			case 1:
				alpha = 0;
		}
		
		return (new Color(new_red,new_green,new_blue,alpha)).getRGB();
	}
	
	public BufferedImage getBlendedImage(Color c,float opacity){
		BufferedImage bi = new BufferedImage(width,height,TYPE_INT_ARGB);
		renderImage(bi);
		for(int y_counter = 0;y_counter < height;y_counter++){
			for(int x_counter = 0;x_counter < width;x_counter++){
				bi.setRGB(x_counter,y_counter,blendPixel(x_counter,y_counter,bi.getRGB(x_counter,y_counter),c,opacity));
			}
		}
		
		return bi;
	}
	
	public ImageIcon getBlendedImageIcon(Color c,float opacity){
		return (new ImageIcon(getBlendedImage(c,opacity)));
	}
	
	public Image getGrayscaledImage(){
		return GrayFilter.createDisabledImage(getImage());
	}
	
	public ImageIcon getGrayscaledImageIcon(){
		return (new ImageIcon(getGrayscaledImage()));
	}
	
	public BufferedImage makeTransparency(BufferedImage bi){
		renderImage(bi);
		for(int y_counter = 0;y_counter < height;y_counter++){
			for(int x_counter = 0;x_counter < width;x_counter++){
				if(andData[y_counter*width + x_counter] == 1){
					Color c = new Color(bi.getRGB(x_counter,y_counter));
					bi.setRGB(x_counter,y_counter,(new Color(c.getRed(),c.getGreen(),c.getBlue(),0).getRGB()));
				}
			}
		}
		
		return this;
	}
	
	public ImageIcon getImageIcon(){
		return (new ImageIcon(getImage()));
	}
	
	public BufferedImage getImage(){
		if(!rendered){ makeTransparency(this); rendered = true; }
		return this;
	}
	
	public void renderImage(BufferedImage bi){
		switch(infoBitCount){
			case 1:
				getOneBitImage(bi,xorData);
				break;
			case 4:
			case 8:
			for(int y_counter = 0;y_counter < height;y_counter++)
				for(int x_counter = 0;x_counter < width;x_counter++){
					bi.setRGB(x_counter,y_counter,rgbPallete[xorData[(height - y_counter - 1)*width + x_counter]].getRGB());
				}
				break;
		}
	}
	
	public BufferedImage getANDImage(){
		BufferedImage bi = new BufferedImage(width,height,TYPE_INT_ARGB);
		getOneBitImage(bi,andData);
		return bi;
	}
	
	private void getOneBitImage(BufferedImage bi,int[] data){
		for(int y_counter = 0;y_counter < height;y_counter++){
			for(int x_counter = 0;x_counter < width;x_counter++){
				bi.setRGB(x_counter,y_counter,data[y_counter*width + x_counter] == 0 ? Color.black.getRGB() : Color.white.getRGB());
			}	
		}
	}
	
	public String getInfoHeader(){
		return ("InfoHeader - Width:" + infoWidth + " Height:" + infoHeight + " BitCount:" + infoBitCount);
	}
	
	public void loadIcon() throws Exception{
		CDataInputStream cdis = new CDataInputStream(new FileInputStream(iconFile));
		cdis.skipBytes(offset);
		readInfoHeader(cdis);
		switch(infoBitCount){
			case 1:
				read1bit(cdis);
				break;
			case 4:
				readRGB(cdis);
				read4bit(cdis);
				break;
			case 8:
				readRGB(cdis);
				read8bit(cdis);
				break;
			default:
				cdis.close();
				throw new Exception("Undefined bitCount " + infoBitCount);
		}
		cdis.close();
	}
	
	private void readInfoHeader(CDataInputStream cdis) throws Exception{
		infoSize            = (int)cdis.readReversedUnsignedInt();
		infoWidth           = (int)cdis.readReversedUnsignedInt();
		infoHeight          = (int)cdis.readReversedUnsignedInt();
		infoPlanes          = cdis.readReversedUnsignedShort();
		infoBitCount        = cdis.readReversedUnsignedShort();
		infoCompression     = (int)cdis.readReversedUnsignedInt();
		infoImageSize       = (int)cdis.readReversedUnsignedInt();
		infoXPixelsPerM     = (int)cdis.readReversedUnsignedInt();
		infoYPixelsPerM     = (int)cdis.readReversedUnsignedInt();
		infoColorsUsed      = (int)cdis.readReversedUnsignedInt();
		infoColorsImportant = (int)cdis.readReversedUnsignedInt();
	}
	
	private void readRGB(CDataInputStream cdis) throws Exception{
		rgbPallete = new Color[colorCount];
		for(int counter = 0;counter < colorCount;counter++){
			int[] rgb = cdis.readUnsignedBytes(4);
			rgbPallete[counter] = new Color(rgb[2],rgb[1],rgb[0]);
		}
	}
	
	private void read1bit(CDataInputStream cdis) throws Exception{
		xorData = readOneBitData(cdis);
		andData = readOneBitData(cdis);
	}
	
	private void read4bit(CDataInputStream cdis) throws Exception{
		xorData = readFourBitData(cdis);
		andData = readOneBitData(cdis);
	}

	private void read8bit(CDataInputStream cdis) throws Exception{
		xorData = cdis.readUnsignedBytes(width*height);
		andData = readOneBitData(cdis);
	}
	
	private int[] readOneBitData(CDataInputStream cdis) throws Exception{
		int[] result = new int[width*height];
		
		for(int lineCounter = 0;lineCounter < height;lineCounter++){
			int lineBuf = cdis.readInt();
			char[] chBuf = getBinaryString(lineBuf).toCharArray();
			
			for(int resultCounter = 0;resultCounter < width;resultCounter++){
				if(chBuf[resultCounter] == '0'){ result[(height - lineCounter - 1)*width + resultCounter] = 0; }
				else{ result[(height - lineCounter - 1)*width + resultCounter] = 1; }
			}
		}
		
		return result;
	}
	
	private int[] readFourBitData(CDataInputStream cdis) throws Exception{
		int[] result = new int[width*height];
		
		for(int lineCounter = 0;lineCounter < height;lineCounter++){
			for(int resultCounter = 0;resultCounter < width;resultCounter+=2){
				String bStr = getBinaryByte(cdis.readUnsignedByte());
				
				result[lineCounter*width + resultCounter] = getQuartet(bStr,1);
				result[lineCounter*width + resultCounter + 1] = getQuartet(bStr,2);
			}
		}
		
		
		return result;
	}
	
	private String getBinaryByte(int b){
		StringBuffer strbuf = new StringBuffer();
		for(int counter = 0;counter < 8;counter++){
			strbuf.append(b % 2);
			b /= 2;
		}
		strbuf.reverse();
		
		return strbuf.toString();
	}
	
	private int getQuartet(String str,int pos){
		String substr;
		
		switch(pos){
			case 1:
				substr = str.substring(0,4);
				break;
			case 2:
				substr = str.substring(4,8);
				break;
			default:
				substr = null;
		}
		
		return Integer.parseInt(substr,2);
	}
		
	private String getBinaryString(int n){
		String str = Integer.toBinaryString(n);
		StringBuffer strResult = new StringBuffer();
		if(str.length() < 32){
			for(int counter = 0;counter < (32 - str.length());counter++){
				strResult.append("0");
			}
		}
		strResult.append(str);
		
		return strResult.toString();
	}
					
	/* static */
	public static Win32Icon loadIcon(CDataInputStream dis,File fl) throws Exception{
		//
		int bWidth        = dis.readUnsignedByte();
		int bHeight       = dis.readUnsignedByte();
		int bColorCount   = dis.readUnsignedByte();
		int bReserved     = dis.readUnsignedByte();
		int wPlanes       = dis.readReversedUnsignedShort();
		int wBitCount     = dis.readReversedUnsignedShort();
		int dwBytesInRes  = (int)dis.readReversedUnsignedInt();
		int dwImageOffset = (int)dis.readReversedUnsignedInt();
		// Entries
		Win32Icon result = new Win32Icon(bWidth,bHeight);
		result.iconFile = fl;
		result.width  = bWidth;
		result.height = bHeight;
		switch(bColorCount){
			case 0:
				result.colorCount = 256;
				break;
			case 2:
				result.colorCount = 2;
				break;
			default:
				result.colorCount = 16;			
		}
		result.bitCount = wBitCount;
		result.planes = wPlanes;
		result.byteSize = dwBytesInRes;
		result.offset = dwImageOffset;
		
		return result;
	}
}