package renegade.gfx.win32.ico;

import java.io.*;
import java.util.Vector;

import renegade.gfx.io.*;

public class Win32IconResource{
	private int iconCount;
	private Vector win32Icons;
	private File fileIcon;

	/* Constructors */
	public Win32IconResource(){
		super();
		win32Icons = new Vector();
		fileIcon = new File("None assigned");
	}
	
	/* overloading */
	public String toString(){
		return ("Win32IconResource (" + fileIcon + "), found " + iconCount + " icons");
	}
	
	public File getFileIcon(){
		return fileIcon;
	}
	
	public Win32Icon getWin32Icon(int idx){
		return (Win32Icon)win32Icons.get(idx);
	}
	
	public Win32Icon[] getWin32Icon(){
		Win32Icon[] result = new Win32Icon[iconCount];
		for(int counter = 0;counter < iconCount;counter++){
			result[counter] = (Win32Icon)win32Icons.get(counter);
		}
		
		return result;
	}
	
	/* accessors & mutators */
	public int getIconCount(){
		return iconCount;
	}
	
	/* read headers & data functions */
	private void readIconHeaders(CDataInputStream dis) throws Exception{
		win32Icons.clear();
		// IconHeader
		int idReserved = dis.readReversedUnsignedShort();
		int idType = dis.readReversedUnsignedShort();
		int idCount = dis.readReversedUnsignedShort();
		if(idType != 1){ 
			System.err.println("Win32Icon::readIconHeaders(DataOutputStream) -- Invalid Icon Headers");
		}
		if(idCount <= 0){
			System.err.println("Win32Icon::readIconHeaders(DataOutputStream) -- Invalid Icon Count");
			throw (new Exception("Invalid Icon Count"));
		}
		iconCount = idCount;
		// IconDirEntry
		for(int counter = 0;counter < idCount;counter++){
			win32Icons.add(Win32Icon.loadIcon(dis,fileIcon));
		}
	}
	
	/* static methods */
	public static Win32IconResource loadIcon(String resource){
		return loadIcon(new File(resource));
	}
	
	public static Win32IconResource loadIcon(File file){
		try{
			if(!file.exists()){ 
				throw (new Exception("File does not exist (" + file.toString() + ")"));
			}
			Win32IconResource result = new Win32IconResource();
			result.fileIcon = file;
			CDataInputStream dis = new CDataInputStream(new FileInputStream(file));
			result.readIconHeaders(dis);
			dis.close();
			return result;
		}catch(Exception e){
			System.err.println(e.toString());
			return null;
		}
	}
}